/* gunmetal is just like Entropy's "brushedmetal.sl" shader, but
 * adjusts the color for the grain pattern, and also adds environment
 * reflections.
 */

#include "filterwidth.h"
#include "noises.h"
#include "material.h"
#include "project.h"
#include "reflections.h"


surface gunmetal ( float Ka = 1, Kd = 0.1, Ks = .9;
		   float uroughness = 0.2, vroughness = 0.5;
		   float graintype = 1;
		   float grainfreq = 400;
		   float grainamp = .75;
		   vector graindir = vector "shader" (0,0,1);
		   string envname = "reflection";
		   string envspace = "current";
		   float envrad = 0;
		   float blur = 0.0;
		   float envsamples = 1;
		   float Kr = .125;
    )
{
    point Pgrain;
    if (graintype == 1) {
	Pgrain = point (s, t, 1);
    } else {
	/* default */
	point Pshad = transform("shader", P);
	Pgrain = Pshad;
    }
    Pgrain *= grainfreq;
    Pgrain *= (.025,1,1);

    float dgrain = filterwidthp(Pgrain);
    float grain = 0.5 + (grainamp/2) * filteredsnoise(Pgrain, dgrain);

    normal Nf = faceforward (normalize(N), I);
    Ci = MaterialBrushedMetal (Nf, Cs*grain, Ka, Kd, Ks, 
			       normalize(dPdu), uroughness, vroughness);
    Ci += Environment (envname, reflect(normalize(I),Nf), envspace, envrad,
		       blur, envsamples, Kr);
}
