/************************************************************************
 * ambocclude.sl
 *
 * Description:
 *   Determines the "ambient occlusion" -- how much of the hemisphere
 *   at each surface point is occluded by objects in the scene.
 *   The surface color is set to black where completely occluded,
 *   white if completely unoccluded.  In addition, the output variable
 *   Nunocc is set to the average normal of the unoccluded samples.
 *   The various computations build in the cosine weighting on the 
 *   hemisphere.
 * 
 * Parameters:
 *   maxhitdist - causes ray hits farther than this distance to be ignored
 *      by the occlusion test.
 *   bias - ignores ray hits closer than this amount, handy for preventing
 *      incorrect self-intersections due to numeric tolerance problems.
 *      If bias < 0, it'll use the global Option "shadow" "bias".
 *
 * Output parameters:
 *   Nunocc - contains the average unoccluded direction.
 *
 * Entropy and BMRT are:
 * (c) Copyright 2001 Exluna, Inc.  Portions (c) 1990-2000 Larry Gritz.
 * All rights reserved.
 ************************************************************************/


surface ambocclude (float maxhitdist = 1.0e6;
		    float bias = -1;
		    output varying normal Nunocc = 0;)
{
    normal Nf = normalize (faceforward(N, I));
    float a = 1 - occlusion (P, Nf, PI/2, Nunocc, "maxhitdist", maxhitdist,
			     "bias", bias);
    Ci = a;
}
