/************************************************************************
 * envlight.sl
 *
 * Description:
 *   A light that illuminates based on an environment map, taking
 *   local occlusion into account (assuming that objects are tagged as
 *   visible in ray tracing).  Works best if the environment map is
 *   high dynamic range.
 * 
 * Parameters:
 *   envname - name of the environment map
 *   envspace - space in which to look up the direction
 *   blur - how much to blur the map
 *   intensity - scale the brightness of the map lookup
 *   maxhitdist - causes ray hits farther than this distance to be ignored
 *      by the occlusion test.
 *   bias - ignores ray hits closer than this amount, handy for preventing
 *      incorrect self-intersections due to numeric tolerance problems.
 *      If bias < 0, it'll use the global Option "shadow" "bias".
 *
 * Entropy and BMRT are:
 * (c) Copyright 2001 Exluna, Inc.  Portions (c) 1990-2000 Larry Gritz.
 * All rights reserved.
 ************************************************************************/


light envlight (string envname = "";
		string envspace = "shader";
		float blur = 0.5;
		float intensity = 1;
		float maxhitdist = 1.0e6;
		float bias = -1;)
{
    vector Nunocc;
    float a = 1 - occlusion (Ps, Ns, PI/2, Nunocc, "maxhitdist", maxhitdist,
			     "bias", bias);
    solar (-Nunocc, 0) {
	Cl = 0;
	if (envname != "") {
	    vector D = normalize(vtransform(envspace, Nunocc));
	    if (a > 0)
		Cl = a * color environment (envname, D, "blur", blur);
	}
    }
    Cl *= intensity;
}
