/* hair.sl - simple shader appropriate for hair using Curves primitives.
 *
 * Parameters:
 *    Ka, Kd, Ks, roughness -- the usual meaning
 *    rootcolor - the color of the hair near the root (v=0)
 *    tipcolor - the color of the hair near the tip (v=1)
 *
 * Hints:
 *    * It looks better if tipcolor is a bit brighter than rootcolor.
 *
 * Entropy and BMRT are:
 * (c) Copyright 2001 Exluna, Inc.  Portions (c) 1990-2000 Larry Gritz.
 * All rights reserved.
 */

#include "locillum.h"

surface hair (float Ka = 1, Kd = .6, Ks = 0.15, roughness = .025;
	      varying color rootcolor = color (.109, .037, .007);
	      varying color tipcolor = color (.519, .325, .125);)
{
    color hcolor = mix (rootcolor, 0.5*tipcolor, v*v);
    color specularcolor = (color(1,1,1)+hcolor) / 2;

    vector T = normalize (dPdv);
    normal Nf = faceforward(normalize(N),I);
    vector V = -normalize(I);

    Ci = hcolor * (Ka*ambient() + Kd*LocIllumHairDiffuse(T))
	+ Ks*specularcolor*LocIllumHairSpecular(T,V,roughness);
    Oi = Os;
    Ci *= Oi;
}
