/* hemilight.sl - simple hemispherical light
 *
 * Description:
 *   This is really just a shadowed distantlight with an adjustable
 *   angle that the light subtends. The name of this shader is a
 *   misnomer -- while the default angle causes light to appear to
 *   come from a perfect hemisphere centered around the "up" vector,
 *   in fact the user can adjust the angle.
 *
 * Parameters:
 *   intensity - basic scale on the intensity of the light
 *   lightcolor - color of the light emitted
 *   up - vector giving the "up" axis of the illumination angle
 *   shadowname - the name of the shadow map to use, if any
 *   shadowsamples - how many samples for shadow map sampling (larger is more
 *        expensive but higher quality)
 *   shadowblur - extra additive blur for shadow edges
 *   shadowbias - extra lookup bias to eliminate precision artifacts.
 *        Bias < 0 means use the global value set by Option "shadow" "bias"
 *   
 * Hints:
 *   To use ray traced shadows for this light to ray trace, just pass
 *   shadowname="shadow".
 *
 * Entropy and BMRT are:
 * (c) Copyright 2001 Exluna, Inc.  Portions (c) 1990-2000 Larry Gritz.
 * All rights reserved.  */

light
hemilight ( float intensity = 1;
	    color lightcolor = 1;
	    vector up = vector "shader" (0,0,1);
	    float  angle = PI/2;
	    string shadowname = "";
	    float  shadowsamples = 1;
	    float  shadowblur = 0;
	    float  shadowbias = -1; )
{
    solar (-up, angle) {
	Cl = intensity * lightcolor;
        if (shadowname != "")
            Cl *= 1 - color shadow (shadowname, Ps, "samples", shadowsamples,
				    "blur", shadowblur, "bias", shadowbias);
    }
}
