/*
 * lumpy.sl -- simple noise bump
 *
 * DESCRIPTION:
 *   Makes a surface appear slightly lumpy
 * 
 * Parameters:
 *   Km - the amplitude of the dents
 *   power - controls the fractal dimension of the dents (1 looks like
 *           previously crumpled paper or cloth, 3 looks like worn metal).
 *   shadingspace - the space that the dents (and their amplitudes) are
 *                  are defined in (default: "shader")
 *   shadingfreq - the frequency of the dents
 *   Pref - if supplied, a reference mesh
 *
 * Entropy and BMRT are:
 * (c) Copyright 2001 Exluna, Inc.  Portions (c) 1990-2000 Larry Gritz.
 * All rights reserved.
 */


#include "noises.h"
#include "displace.h"
#define USE_PREF 1
#include "pshad.h"



displacement
lumpy ( float Km = 1, power = 1, maxoctaves = 3;
	PSHAD_PARAMS("shader",1); )
{
    GET_PSHAD;
    float magnitude = fBm (Pshad, dPshad, maxoctaves, 2, 0.5);
    magnitude = pow (magnitude, power);
    Displace (shadingspace, Km*magnitude);
}
