/* pointlight.sl - simple point light source
 *
 * Parameters:
 *   intensity - basic scale on the intensity of the light
 *   lightcolor - color of the light emitted
 *   from - specifies the position of the light cone
 *   falloff - the distance falloff exponent (2==1/r^2, 1==1/r, 0==no falloff)
 *   shadowname - the name of the shadow map to use, if any
 *   shadowsamples - how many samples for shadow map sampling (larger is more
 *        expensive but higher quality)
 *   shadowblur - extra additive blur for shadow edges
 *   shadowbias - extra lookup bias to eliminate precision artifacts
 *        Bias < 0 means use the global value set by Option "shadow" "bias"
 *   
 * Hints:
 *   To use ray traced shadows for this light to ray trace, just pass
 *   shadowname="shadow".
 *
 * Entropy and BMRT are:
 * (c) Copyright 2001 Exluna, Inc.  Portions (c) 1990-2000 Larry Gritz.
 * All rights reserved.  
 */

light
pointlight (float intensity = 1;
	    color lightcolor = 1;
	    point from = point "shader" (0,0,0);
	    float falloff = 2;
	    string shadowname = "";
	    float  shadowsamples = 1;
	    float  shadowblur = 0;
	    float  shadowbias = -1; )
{
    illuminate (from) {
	Cl = intensity * lightcolor / pow(length(L), falloff);
        if (shadowname != "")
            Cl *= 1 - color shadow (shadowname, Ps, "samples", shadowsamples,
				    "blur", shadowblur, "bias", shadowbias);
    }
}
